@@ -1,6 +1,7 @@
-#' @title Sort Pairs
+#' Sort Pairs
+#'
+#' A function to sort pairs of integers or factors and identify the pairs
 #'
-#' @description A function to sort pairs of integers or factors and identify the pairs
 #' @param c1 a vector of length n with value between 0 and N1 < n
 #' @param c2 a vector of length n with value between 0 and N2 < n
 #' @param spMat logical: send back the contingency table as sparsely encoded (cost more than the algorithm itself). Default is FALSE
@@ -20,15 +21,13 @@ sortPairs <- function(c1, c2, spMat=FALSE){
 
   ## if c1 and c2 are integer
   if (is.integer(c1) & is.integer(c2)) {
-    mylevels <- list(c1 = unique(c1), c2 = unique(c2))
-    c1 <- c1 - min(c1)
-    c2 <- c2 - min(c2)
-    ## if the range is not adapted to the C code
-    if (!(max(c1) <= n-1 & max(c2) <= n-1)) {
-      c1 <- as.integer(factor(c1, levels = mylevels$c1)) - 1L
-      c2 <- as.integer(factor(c2, levels = mylevels$c2)) - 1L
-    }
-    ## if factor, force conversion to integer
+    ## getRank is O(n) if max(c1)-min(c1) and max(c2)-min(c2) is of order length(c1)=length(c2)
+    ## NOTE: getRank does not assume c1 and c2 are between 0 and n
+    res1 <- getRank(c1)
+    res2 <- getRank(c2)
+    mylevels <- list(c1=res1$index, c2=res2$index)
+    c1 <- res1$translated  # here ranks are in [0, n)
+    c2 <- res2$translated  # here ranks are in [0, n)
   } else if (is.factor(c1) & is.factor(c2)) {
     mylevels <- list(c1 = levels(c1), c2 = levels(c2))
     c1 <- as.integer(c1) - 1L
@@ -58,13 +57,16 @@ sortPairs <- function(c1, c2, spMat=FALSE){
               levels = mylevels,
               nij = out$pair_nb,
               ni. = out$c1_nb,
-              n.j = out$c2_nb
+              n.j = out$c2_nb,
+              pair_c1 = out$pair_c1,
+              pair_c2 = out$pair_c2
   )
   res
 }
 
-#' @title Adjusted Rand Index
-#' @description A function to compute the adjusted rand index between two classifications
+#' Adjusted Rand Index
+#'
+#' A function to compute the adjusted rand index between two classifications
 #'
 #' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
 #' @param c2 a vector containing the labels of the second classification.
@@ -99,8 +101,9 @@ ARI <- function(c1, c2){
     res
 }
 
-#' @title Rand Index
-#' @description A function to compute the rand index between two classifications
+#' Rand Index
+#'
+#' A function to compute the rand index between two classifications
 #'
 #' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
 #' @param c2 a vector containing the labels of the second classification.
@@ -124,8 +127,117 @@ RI <- function(c1, c2){
   res
 }
 
-#' @title Entropy
-#' @description A function to compute the empirical entropy for two vectors of classification and the joint entropy
+#' Modified Adjusted Rand Index
+#'
+#' A function to compute a modified adjusted rand index between two classifications as proposed by Sundqvist et al. in prep, based on a multinomial model.
+#'
+#' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
+#' @param c2 a vector containing the labels of the second classification.
+#' @return a scalar with the modified ARI.
+#' @seealso \code{\link{ARI}}, \code{\link{NID}}, \code{\link{NVI}}, \code{\link{NMI}}, \code{\link{clustComp}}
+#' @examples
+#' data(iris)
+#' cl <- cutree(hclust(dist(iris[,-5])), 4)
+#' MARI(cl,iris$Species)
+#' @export
+MARI <- function(c1, c2){
+  ## get pairs using C
+  ## ensure that values of c1 and c2 are between 0 and n1
+  res <- sortPairs(c1, c2)
+  N <- length(c1)
+  ##
+
+  stot <- sum(choose(res$nij, 2), na.rm=TRUE)
+  srow <- sum(choose(res$ni., 2), na.rm=TRUE)
+  scol <- sum(choose(res$n.j, 2), na.rm=TRUE)
+
+  ## using Lemma 3.3
+  ## triplets
+  T1 <- 2*N
+  T2 <- sum(res$nij * res$ni.[res$pair_c1+1] * res$n.j[res$pair_c2+1], na.rm=TRUE)
+  T3 <- -sum(res$nij^2, na.rm=TRUE) - sum(res$ni.^2, na.rm=TRUE) - sum(res$n.j^2, na.rm=TRUE)
+
+  ## quadruplets (and division by 6 choose(N, 4)
+  expectedIndex <- (srow*scol - stot - (T1+T2+T3)) / (6 *choose(N, 4))
+
+  ## return the rand-index
+  expectedIndex <- expectedIndex * choose(N, 2) ## RESCALE SO THAT THE CODE IS EQUIVALENT TO THE ARI
+  maximumIndex <- (srow+scol)/2
+  if (expectedIndex == maximumIndex & stot != 0) {
+    res <- 1
+  } else {
+    res <- (stot-expectedIndex)/(maximumIndex-expectedIndex)
+  }
+    res
+  ## return the adjusted (and divided) rand-index
+  res
+}
+
+#' raw Modified Adjusted Rand Index
+#'
+#' A function to compute a modified adjusted rand index between two classifications as proposed by Sundqvist et al. in prep, based on a multinomial model. Raw means, that the index is not divided by the (maximum - expected) value.
+#'
+#' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
+#' @param c2 a vector containing the labels of the second classification.
+#' @return a scalar with the modified ARI without the division by the (maximum - expected)
+#' @seealso \code{\link{ARI}}, \code{\link{NID}}, \code{\link{NVI}}, \code{\link{NMI}}, \code{\link{clustComp}}
+#' @examples
+#' data(iris)
+#' cl <- cutree(hclust(dist(iris[,-5])), 4)
+#' MARIraw(cl,iris$Species)
+#' @export
+MARIraw <- function(c1, c2){
+  ## get pairs using C
+  ## ensure that values of c1 and c2 are between 0 and n1
+  res <- sortPairs(c1, c2)
+  N <- length(c1)
+
+  stot <- sum(choose(res$nij, 2), na.rm=TRUE)
+  srow <- sum(choose(res$ni., 2), na.rm=TRUE)
+  scol <- sum(choose(res$n.j, 2), na.rm=TRUE)
+
+  ## using Lemma 3.3
+  ## triplets
+  T1 <- 2*N
+  T2 <- sum(res$nij * res$ni.[res$pair_c1+1] * res$n.j[res$pair_c2+1], na.rm=TRUE)
+  T3 <- -sum(res$nij^2, na.rm=TRUE) - sum(res$ni.^2, na.rm=TRUE) - sum(res$n.j^2, na.rm=TRUE)
+
+  ## quadruplets (and division by 6 choose(N, 4)
+  expectedIndex <- (srow*scol - stot - (T1+T2+T3)) / (6 *choose(N, 4))
+
+  ## return the rand-index
+  res <- (stot / choose(N, 2)) - expectedIndex
+  res
+}
+
+#' Chi-square statistics
+#'
+#' A function to compute the Chi-2 statistics
+#'
+#' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
+#' @param c2 a vector containing the labels of the second classification.
+#' @return a scalar with the chi-square statistics.
+#' @seealso \code{\link{ARI}}, \code{\link{NID}}, \code{\link{NVI}}, \code{\link{NMI}}, \code{\link{clustComp}}
+#' @examples
+#' data(iris)
+#' cl <- cutree(hclust(dist(iris[,-5])), 4)
+#' Chi2(cl,iris$Species)
+#' @export
+Chi2 <- function(c1, c2){
+  ## get pairs using C
+  ## ensure that values of c1 and c2 are between 0 and n1
+  res <- sortPairs(c1, c2)
+  N <- length(c1)
+
+  res <- N* sum(res$nij^2 / (res$ni.[res$pair_c1+1] * res$n.j[res$pair_c2+1]) )
+  res <- res - N
+  res
+}
+
+
+#' Entropy
+#'
+#' A function to compute the empirical entropy for two vectors of classification and the joint entropy
 #'
 #' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
 #' @param c2 a vector containing the labels of the second classification.
@@ -148,8 +260,9 @@ entropy <- function(c1, c2){
   res
 }
 
-#' @title measures of similarity between two classification
-#' @description A function various measures of similarity between two classifications
+#' Measures of similarity between two classification
+#'
+#' A function various measures of similarity between two classifications
 #'
 #' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
 #' @param c2 a vector containing the labels of the second classification.
@@ -173,6 +286,7 @@ clustComp <- function(c1, c2) {
 
   EMI <- expected_MI(as.integer(H$ni.), as.integer(H$n.j))
 
+
   res <- list(RI  = RI(c1,c2)             ,
               ARI = ARI(c1,c2)            ,
               MI  = - H$UV + H$U + H$V    ,
@@ -181,13 +295,18 @@ clustComp <- function(c1, c2) {
               NVI = 1 - MI/H$UV           ,
               ID  = max(H$U, H$V) - MI    ,
               NID = 1 - MI / max(H$U, H$V),
-              NMI = MI / max(H$U, H$V)
+              NMI = MI / max(H$U, H$V)    ,
+      	      Chi2 = Chi2(c1,c2)          ,
+  	          MARI = MARI(c1,c2)          ,
+	         MARIraw = MARIraw(c1,c2)
   )
   res
+  res
 }
 
-#' @title Adjusted Mutual Information
-#' @description A function to compute the adjusted mutual information between two classifications
+#' Adjusted Mutual Information
+#'
+#' A function to compute the adjusted mutual information between two classifications
 #'
 #' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
 #' @param c2 a vector containing the labels of the second classification.
@@ -208,8 +327,9 @@ AMI <- function(c1, c2){
   res
 }
 
-#' @title Normalized mutual information (NMI)
-#' @description A function to compute the NMI between two classifications
+#' Normalized mutual information (NMI)
+#'
+#' A function to compute the NMI between two classifications
 #'
 #' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
 #' @param c2 a vector containing the labels of the second classification.
@@ -239,8 +359,9 @@ NMI <- function(c1, c2, variant = c("max", "min", "sqrt", "sum", "joint")) {
   res
 }
 
-#' @title Normalized information distance (NID)
-#' @description A function to compute the NID between two classifications
+#' Normalized information distance (NID)
+#'
+#' A function to compute the NID between two classifications
 #'
 #' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
 #' @param c2 a vector containing the labels of the second classification.
@@ -260,8 +381,10 @@ NID <- function(c1, c2) {
   res
 }
 
-#' @title Normalized variation of information (NVI)
-#' @description A function to compute the NVI between two classifications
+#' Normalized variation of information (NVI)
+#'
+#' A function to compute the NVI between two classifications
+#'
 #' @param c1 a vector containing the labels of the first classification. Must be a vector of characters, integers, numerics, or a factor, but not a list.
 #' @param c2 a vector containing the labels of the second classification.
 #' @return a scalar with the normalized variation of information.
@@ -280,5 +403,3 @@ NVI <- function(c1, c2) {
 }
 
 
-
-
